<?php

namespace Drupal\tmgmt_smartling\Smartling;

use GuzzleHttp\ClientInterface;
use Smartling\Exceptions\SmartlingApiException;

class SmartlingContextApi
{

    const DEFAULT_SERVICE_URL = 'https://api.smartling.com/context-api/v2';

    /**
     * Smartling API base url.
     *
     * @var string
     */
    protected $baseUrl;

    /**
     * Smartling API key.
     *
     * @var string
     */
    protected $apiKey;

    /**
     * Smartling API project ID.
     *
     * @var string
     */
    protected $projectId;

    /**
     * Http Client abstraction.
     *
     * @var \GuzzleHttp\ClientInterface
     */
    protected $httpClient;

    /**
     * Creates SmartlingApi instance.
     *
     * @param string $apiKey
     *   Api Key string.
     * @param string $projectId
     *   Project ID string.
     * @param \GuzzleHttp\ClientInterface $http_client
     *   Instance of Guzzle http client.
     * @param string $base_service_url
     *   Url for Smartling translation service.
     */
    public function __construct($apiKey, $projectId, ClientInterface $http_client, $base_service_url = null)
    {
        $this->apiKey = $apiKey;
        $this->projectId = $projectId;
        $this->httpClient = $http_client;
        $this->baseUrl = rtrim($base_service_url ?: self::DEFAULT_SERVICE_URL , '/');
    }

    /**
     * Sends request to Smartling Service via Guzzle Client.
     *
     * @param string $uri
     *   Resource uri.
     * @param array $requestData
     *   Parameters to be send as query or multipart form elements.
     * @param string $method
     *   Http method uppercased.
     *
     * @param array $filenames
     * @return array Decoded JSON answer.
     * Decoded JSON answer.
     * @throws \Smartling\Exceptions\SmartlingApiException
     */
    protected function sendRequest($uri, $requestData, $method, $filenames = [])
    {
        // Set api key and product id as required arguments.
        $requestData['apiKey'] = $this->apiKey;
        $requestData['projectId'] = $this->projectId;

        // Ask for JSON and disable Guzzle exceptions.
        $options = [
          'headers' => [
            'Accept' => 'application/json',
          ],
          'http_errors' => FALSE,
        ];

        // For GET and DELETE http methods use just query parameter builder.
        if (in_array($method, ['GET', 'DELETE'])) {
            $options['query'] = $requestData;
        } else {
            $options['multipart'] = [];
            // Remove file from params array and add it as a stream.
            if (!empty($requestData['file'])) {
                $options['multipart'][] = [
                  'name' => 'file',
                  'contents' => $this->readFile($requestData['file']),
                ];
                unset($requestData['file']);
            }
            foreach ($requestData as $key => $value) {
                // Hack to cast FALSE to '0' instead of empty string.
                if (is_bool($value)) {
                    $value = (int)$value;
                }

                $val = [
                  'name' => $key,
                  // Typecast everything to string to avoid curl notices.
                  'contents' => (string) $value,
                ];
                if (isset($filenames[$key])) {
                    $val['filename'] = $filenames[$key]['name'];
                    $val['Content-Type'] = $filenames[$key]['content_type'];
                }

                $options['multipart'][] = $val;
            }
        }

        // Avoid double slashes in final URL.
        $uri = ltrim($uri, "/");

        $guzzle_response = $this->httpClient->request($method, $this->baseUrl . '/' . $uri, $options);
        $response_body = (string) $guzzle_response->getBody();

        // Catch all errors from Smartling and throw appropriate exception.
        if ($guzzle_response->getStatusCode() >= 400) {
            $error_response = json_decode($response_body, TRUE);

            $error_messages = [];
            if (!empty($error_response['response']['errors'])) {
                $error_messages = $error_response['response']['errors'];
            } elseif (!empty($error_response['response']['messages'])) {
                $error_messages = $error_response['response']['messages'];
            }
            if (!$error_response || empty($error_messages)) {
                throw new SmartlingApiException('Bad response format from Smartling');
            }

            throw new SmartlingApiException(print_r($error_messages, TRUE), $guzzle_response->getStatusCode());
        }

        // "Download file" method return translated file directly.
        if ('file/get' == $uri) {
            return $response_body;
        }
        $response = json_decode($response_body, TRUE);
        // Throw exception if json is not valid.
        if (!$response || !isset($response['response']['data'])) {
            throw new SmartlingApiException('Bad response format from Smartling');
        }

        return $response['response']['data'];
    }

    /**
     * upload context to Smartling
     *
     * @param array $data
     * @return string
     */
    public function uploadContext($data, $filenames)
    {
        $orgID = $data['orgId'];
        unset($data['orgId']);

        return $this->sendRequest('js/' . $orgID . '/html', $data, 'POST', $filenames);
    }

    public function putResource($data, $filenames) {
        $orgID = $data['orgId'];
        unset($data['orgId']);

        return $this->sendRequest('js/projects/' . $orgID . '/resources/' . $data['resourceId'], $data, 'PUT', $filenames);
    }

}
