<?php

/**
 * @file
 * Contains \Drupal\tmgmt_smartling\Tests\SmartlingTest.
 */

namespace Drupal\tmgmt_smartling\Tests;

use Drupal;
use Drupal\node\Entity\Node;
use Drupal\tmgmt\Tests\TMGMTTestBase;
use Smartling\Exceptions\SmartlingApiException;

/**
 * Basic tests for the Smartling translator.
 *
 * @group tmgmt_smartling
 */
class SmartlingTest extends TMGMTTestBase {

  /**
   * Name of file that contains settings for test Smartling project.
   *
   * @var string
   */
  const SETTINGS_FILE_NAME = 'tmgmt_smartling.simpletest.settings.php';

  /**
   * Smartling test project settings.
   *
   * @var array
   */
  private $smartlingPluginProviderSettings = [];

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'tmgmt',
    'tmgmt_demo',
    'tmgmt_smartling',
    'tmgmt_smartling_context_debug',
    'tmgmt_smartling_test',
    'dblog',
  ];

  /**
   * @var int
   */
  private $testNodeId = 3;

  /**
   * @var string
   */
  private $testNodeTitle;

  /**
   * @var string
   */
  private $testNodeBody;

  /**
   * @var int
   */
  private $testJobId = 1;

  /**
   * @var string
   */
  private $generatedFileName;

  /**
   * @var string
   */
  private $targetLanguage = 'fr';

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    $settings_file_path = __DIR__ . '/../../' . self::SETTINGS_FILE_NAME;

    // Include settings from external file.
    if (file_exists($settings_file_path) && empty($this->smartlingPluginProviderSettings)) {
      require_once $settings_file_path;

      // Fetch needed data.
      $this->smartlingPluginProviderSettings = $settings;
      $extension = $this->smartlingPluginProviderSettings['settings[export_format]'] == 'xlf' ? 'xlf' : 'xml';
      $test_node = Node::load($this->testNodeId);
      $this->testNodeTitle = $test_node->get('title')->value;
      $this->testNodeBody = trim(strip_tags($test_node->get('body')->value));
      $this->generatedFileName = "JobID{$this->testJobId}_en_{$this->targetLanguage}.{$extension}";
    }

    $this->deleteTestFile($this->generatedFileName);

    // Additional permission: access to "Recent log messages" page and
    // access for manual context sending.
    $this->loginAsAdmin([
      'access site reports',
      'send context smartling',
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function tearDown() {
    parent::tearDown();

    $this->deleteTestFile($this->generatedFileName);
  }

  /**
   * Removes test file from Smartling dashboard.
   *
   * @param $fileName
   */
  private function deleteTestFile($fileName) {
    try {
      $api_factory = Drupal::service('tmgmt_smartling.smartling_api_factory');
      $smartlingApi = $api_factory::create([
        'user_id' => $this->smartlingPluginProviderSettings['settings[user_id]'],
        'project_id' => $this->smartlingPluginProviderSettings['settings[project_id]'],
        'token_secret' => $this->smartlingPluginProviderSettings['settings[token_secret]'],
      ], 'file');
      $smartlingApi->deleteFile($fileName);
    }
    catch (SmartlingApiException $e) {
      // File not found.
    }
  }

  /**
   * Sets up Smartling provider settings.
   *
   * @param array $providerSettings
   * @param string $assertText
   * @throws \Exception
   */
  private function setUpSmartlingProviderSettings(array $providerSettings, $assertText = 'Smartling translator configuration has been updated.') {
    $this->drupalPostForm('admin/tmgmt/translators/manage/smartling', $providerSettings, t('Save'));
    $this->assertText($assertText);
  }

  /**
   * Requests translation for a given node.
   *
   * @param $nid
   * @param $language
   * @param string $assertText
   * @throws \Exception
   */
  private function requestTranslationForNode($nid, $language, $assertText = 'Exported file can be downloaded here.') {
    $data = [
      "items[$nid]" => "item[$nid]",
    ];
    $this->drupalPostForm('admin/tmgmt/sources', $data, t('Request translation'));
    $this->assertText(t('One job needs to be checked out.'));

    $data = [
      'label[0][value]' => 'Test job',
      'target_language' => $language,
      'translator' => 'smartling',
    ];
    $this->drupalPostForm("admin/tmgmt/jobs/$this->testJobId", $data, t('Submit to provider'));
    $this->assertText($assertText);
  }

  /**
   * Checks if generated file exists and correct.
   *
   * @param $fileName
   * @param $nodeTitle
   * @param $nodeBody
   */
  private function checkGeneratedFile($fileName, $nodeTitle, $nodeBody) {
    $url = \Drupal::service('stream_wrapper_manager')->getViaUri(file_default_scheme() . "://tmgmt_sources/$fileName")->getExternalUrl();
    $this->drupalGet($url);
    $this->assertResponse(200);
    $this->assertRaw($nodeTitle);
  }

  /**
   * Checks if download was successful.
   *
   * @param $jobId
   * @param $fileName
   * @throws \Exception
   */
  private function downloadAndCheckTranslatedFile($jobId, $fileName) {
    $this->drupalPostForm("admin/tmgmt/jobs/$jobId", [], t('Download'));
    $this->drupalGet('admin/reports/dblog');
    $this->assertResponse(200);
    // TODO: don't know why assertLink and assertRaw doesn't work with quoted
    // strings.
    $this->assertRaw('Translation for');
    $this->assertRaw($fileName);
    $this->assertRaw('was successfully downloaded.');
  }

  /**
   * Test Smartling provider plugin form validation.
   */
  public function testSmartlingProviderSettingsFormValidation() {
    // Validation with wrong parameters.
    $this->setUpSmartlingProviderSettings([
      'settings[project_id]' => $this->randomString(),
      'settings[user_id]' => $this->randomString(),
      'settings[token_secret]' => $this->randomString(),
      'settings[key]' => $this->randomString(),
    ], t('The "User Id", "Token Secret", or "Project Id" are not correct.'));

    // Validation with correct parameters
    if (!empty($this->smartlingPluginProviderSettings)) {
      $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test job with misconfigured Smartling provider plugin.
   */
  public function testJobCreationWithMisconfiguredSmartlingProviderPlugin() {
    // Check that "First node" provided by tmgmt_demo module is presented
    // at "Sources" page.
    $this->drupalGet('admin/tmgmt/sources');
    $this->assertResponse(200);
    $this->assertText($this->testNodeTitle, 'Test node "First node" is found.');

    // Request translation for "First node" (nid 3) node for fr locale.
    $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage, t('@translator is not available. Make sure it is properly configured.', ['@translator' => 'Smartling translator']));
  }

  /**
   * Test upload and download translation.
   */
  public function testUploadFileAndDownloadTranslation() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);
      $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage);
      $this->checkGeneratedFile($this->generatedFileName, $this->testNodeTitle, $this->testNodeBody);

      // Check fr node title before translation (should be same as en title).
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertText($this->testNodeTitle);

      // Download translated file.
      $this->downloadAndCheckTranslatedFile($this->testJobId, $this->generatedFileName);

      // Check translation.
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertNoText($this->testNodeTitle);
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test manual context sending.
   */
  public function testManualContextSending() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);
      $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage);
      $this->drupalPostForm('admin/tmgmt/job_items', [
        'action' => 'tmgmt_smartling_send_context',
        'tmgmt_job_item_bulk_form[0]' => 'WyJ1bmQiLCIxIl0=',
      ], t('Apply to selected items'), [
        'query' => [
          'state' => 'All',
          'source_language' => 'All',
          'target_language' => 'All',
        ],
      ]);
      $this->drupalPostForm(NULL, [], t('Send Context to Smartling'));
      $this->drupalGet('admin/reports/dblog');
      $this->assertRaw('We are about to switch user');
      $this->assertRaw('User was successfully switched');
      $this->assertRaw(t('Context upload for file @filename completed successfully.', ['@filename' => $this->generatedFileName]));
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test context sending by cron.
   */
  public function testContextSendingByCron() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);
      $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage);
      $this->cronRun();
      $this->drupalGet('admin/reports/dblog');
      $this->assertRaw('We are about to switch user');
      $this->assertRaw('User was successfully switched');
      $this->assertRaw(t('Context upload for file @filename completed successfully.', ['@filename' => $this->generatedFileName]));
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test Smartling context debugger: show context.
   */
  public function testSmartlingContextDebuggerShowContext() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      global $base_url;

      $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);

      $this->drupalPostForm('admin/tmgmt/smartling-context-debug', [
        'do_direct_output' => TRUE,
        'url' => $base_url . '/node/' . $this->testNodeId,
      ], t('Test context'));
      $this->assertText($this->testNodeTitle);
      $this->assertText($this->testNodeBody);
      $this->drupalGet('admin/reports/dblog');
      $this->assertNoRaw(t('Context upload for file @filename completed successfully.', ['@filename' => $this->generatedFileName]));
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test Smartling context debugger: send context.
   */
  public function testSmartlingContextDebuggerSendContext() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      global $base_url;

      $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);
      $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage);
      $this->checkGeneratedFile($this->generatedFileName, $this->testNodeTitle, $this->testNodeBody);

      $this->drupalPostForm('admin/tmgmt/smartling-context-debug', [
        'do_direct_output' => FALSE,
        'filename' => $this->generatedFileName,
        'url' => $base_url . '/node/' . $this->testNodeId,
      ], t('Test context'));
      $this->assertNoText($this->testNodeTitle);
      $this->assertNoText($this->testNodeBody);
      $this->assertText('Smartling response');
      $this->drupalGet('admin/reports/dblog');
      $this->assertRaw(t('Context upload for file @filename completed successfully.', ['@filename' => $this->generatedFileName]));
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test context uploading with turned off "Silent switching mode" feature.
   */
  public function testSilentSwitchingOff() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);
      $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage);
      $this->cronRun();
      $this->drupalGet('admin/reports/dblog');
      $this->assertRaw('We are about to switch user');
      $this->assertRaw(t('User @name has logged out.', [
        '@name' => 'Anonymous',
      ]));
      $this->assertRaw(t('User @name has logged in.', [
        '@name' => $this->smartlingPluginProviderSettings['settings[contextUsername]'],
      ]));
      $this->assertRaw('User was successfully switched');
      $this->assertRaw(t('Context upload for file @filename completed successfully.', ['@filename' => $this->generatedFileName]));
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test context uploading with turned on "Silent switching mode" feature.
   */
  public function testSilentSwitchingOn() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      // Set up silent user switching mode.
      $plugin_settings = $this->smartlingPluginProviderSettings;
      $plugin_settings['settings[context_silent_user_switching]'] = TRUE;

      $this->setUpSmartlingProviderSettings($plugin_settings);
      $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage);
      $this->cronRun();
      $this->drupalGet('admin/reports/dblog');
      $this->assertRaw('We are about to switch user');
      $this->assertNoRaw(t('User @name has logged out.', [
        '@name' => 'Anonymous',
      ]));
      $this->assertNoRaw(t('User @name has logged in.', [
        '@name' => $this->smartlingPluginProviderSettings['settings[contextUsername]'],
      ]));
      $this->assertRaw('User was successfully switched');
      $this->assertRaw(t('Context upload for file @filename completed successfully.', ['@filename' => $this->generatedFileName]));
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test push back controller.
   */
  public function testPushBackController() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);
      $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage);
      $this->checkGeneratedFile($this->generatedFileName, $this->testNodeTitle, $this->testNodeBody);

      // Try to push back un-existing job.
      $this->drupalGet("tmgmt-smartling-callback/100500", [
        'query' => [
          'fileUri' => $this->generatedFileName,
          'locale' => 'fr-FR',
        ],
      ]);
      $this->assertResponse(404);

      // Try to push back existing job without locale and fileUri parameters.
      $this->drupalGet("tmgmt-smartling-callback/{$this->testJobId}");
      $this->assertResponse(404);

      // Try to push back existing job without locale parameter.
      $this->drupalGet("tmgmt-smartling-callback/{$this->testJobId}", [
        'query' => [
          'fileUri' => $this->generatedFileName,
        ],
      ]);
      $this->assertResponse(404);

      // Try to push back existing job without fileUri parameter.
      $this->drupalGet("tmgmt-smartling-callback/{$this->testJobId}", [
        'query' => [
          'locale' => 'fr-FR',
        ],
      ]);
      $this->assertResponse(404);

      // Check fr node title before translation (should be same as en title).
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertText($this->testNodeTitle);

      // Try to push back existing job.
      $this->drupalGet("tmgmt-smartling-callback/{$this->testJobId}", [
        'query' => [
          'fileUri' => $this->generatedFileName,
          'locale' => 'fr-FR',
        ],
      ]);
      $this->assertResponse(200);

      // Check fr node title after translation (should be same as en title).
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertNoText($this->testNodeTitle);
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test uploading/downloading xml file: rely on file name in tmgmt_job.
   */
  public function testUploadXmlChangeToXlfDownloadXml() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $providerSettings = $this->smartlingPluginProviderSettings;

      $this->setUpSmartlingProviderSettings($providerSettings);
      $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage);
      $this->checkGeneratedFile($this->generatedFileName, $this->testNodeTitle, $this->testNodeBody);

      // Check fr node title before translation (should be same as en title).
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertText($this->testNodeTitle);

      // Change export format to xlf and download translated file.
      $providerSettings['settings[export_format]'] = 'xlf';
      $this->setUpSmartlingProviderSettings($providerSettings);
      $this->downloadAndCheckTranslatedFile($this->testJobId, $this->generatedFileName);

      // Check translation.
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertNoText($this->testNodeTitle);
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test uploading/downloading xlf file: rely on file name in tmgmt_job.
   */
  public function testUploadXlfChangeToXmlDownloadXlf() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $providerSettings = $this->smartlingPluginProviderSettings;
      $providerSettings['settings[export_format]'] = 'xlf';
      $fileName = "JobID{$this->testJobId}_en_{$this->targetLanguage}.xlf";

      $this->setUpSmartlingProviderSettings($providerSettings);
      $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage);
      $this->checkGeneratedFile($fileName, $this->testNodeTitle, $this->testNodeBody);

      // Check fr node title before translation (should be same as en title).
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertText($this->testNodeTitle);

      // Change export format to xml and download translated file.
      $providerSettings['settings[export_format]'] = 'html';
      $this->setUpSmartlingProviderSettings($providerSettings);
      $this->downloadAndCheckTranslatedFile($this->testJobId, $fileName);

      // Check translation.
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertNoText($this->testNodeTitle);
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

}
